@@ -1,4 +1,4 @@
-{-# LANGUAGE MultiParamTypeClasses, FlexibleContexts, TypeFamilies #-}
+{-# LANGUAGE MultiParamTypeClasses, FlexibleContexts, TypeFamilies, CPP #-}
 ----------------------------------------------------------------------
 -- |
 -- Module      :  Data.AffineSpace
@@ -13,15 +13,26 @@
 
 module Data.AffineSpace
   (
-    AffineSpace(..), (.-^), distanceSq, distance, alerp
+    AffineSpace(..), (.-^), distanceSq, distance, alerp, affineCombo
   ) where
 
 import Control.Applicative (liftA2)
 import Data.Ratio
+import Foreign.C.Types (CFloat, CDouble)
+import Control.Arrow(first)
 
 import Data.VectorSpace
 
-infix 4 .+^, .-^, .-.
+-- Through 0.8.4, I used the following fixities.
+-- 
+--   infix 4 .+^, .-^, .-.
+-- 
+-- Changed in 0.8.5 to match precedence of + and -, and to associate usefully.
+-- Thanks to Ben Gamari for suggesting left-associativity.
+
+infixl 6 .+^, .-^
+infix  6 .-.
+
 
 -- TODO: Convert AffineSpace from fundep to associated type, and eliminate
 -- FunctionalDependencies above.
@@ -56,20 +67,37 @@ alerp :: (AffineSpace p, VectorSpace (Diff p)) =>
          p -> p -> Scalar (Diff p) -> p
 alerp p p' s = p .+^ (s *^ (p' .-. p))
 
-instance  AffineSpace Double where
-  type Diff Double = Double
-  (.-.) =  (-)
-  (.+^) =  (+)
+-- | Compute an affine combination (weighted average) of points.
+-- The first element is used as origin and is weighted
+-- such that all coefficients sum to 1. For example,
+--
+-- > affineCombo a [(0.3,b), (0.2,c)]
+--
+-- is equal to
+--
+-- > a .+^ (0.3 *^ (b .-. a) ^+^ 0.2 *^ (c .-. a))
+--
+-- and if @a@, @b@, and @c@ were in a vector space would also be equal to
+--
+-- > 0.5 *^ a ^+^ 0.3 *^ b ^+^ 0.2 *^ c
+--
+-- See also 'linearCombo' (on vector spaces).
+affineCombo :: (AffineSpace p, v ~ Diff p, VectorSpace v) => p -> [(p,Scalar v)] -> p
+affineCombo z l = z .+^ linearCombo (map (first (.-. z)) l)
 
-instance  AffineSpace Float where
-  type Diff Float = Float
-  (.-.) =  (-)
-  (.+^) =  (+)
+#define ScalarTypeCon(con,t) \
+  instance con => AffineSpace (t) where \
+    { type Diff (t) = t \
+    ; (.-.) = (-) \
+    ; (.+^) = (+) }
 
-instance Integral a => AffineSpace (Ratio a) where
-  type Diff (Ratio a) = Ratio a
-  (.-.) = (-)
-  (.+^) = (+)
+#define ScalarType(t) ScalarTypeCon((),t)
+
+ScalarType(Double)
+ScalarType(CDouble)
+ScalarType(Float)
+ScalarType(CFloat)
+ScalarTypeCon(Integral a,Ratio a)
 
 instance (AffineSpace p, AffineSpace q) => AffineSpace (p,q) where
   type Diff (p,q)   = (Diff p, Diff q)
